package com.xydl.cac.task;

import com.xydl.cac.entity.ModevTypePoint;
import com.xydl.cac.entity.NSensor;
import com.xydl.cac.entity.WarnRule;
import com.xydl.cac.entity.Warning;
import com.xydl.cac.repository.WarnRuleRepository;
import com.xydl.cac.repository.WarningRepository;
import com.xydl.cac.service.DataService;
import com.xydl.cac.socket.WebSocketServer;
import com.xydl.cac.util.DateUtil;
import com.xydl.cac.util.DingTalkPushUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.*;

@Service
@Slf4j
public class RuleCheckTask {
    @Resource
    WarnRuleRepository ruleRepository;
    @Resource
    DataService dataService;
    @Resource
    WarningRepository warningRepository;
    @Resource
    DingTalkPushUtil dingTalkPushUtil;
    @Resource
    WebSocketServer webSocketServer;

    @Scheduled(initialDelay = 60 * 1000, fixedDelay = 3 * 60 * 1000)
    private void checkAll() {
        try {
            Iterator<Integer> it = CacheTask.rule_Cache.keySet().iterator();
            while (it.hasNext()) {
                Integer key = it.next();
                WarnRule rule = CacheTask.rule_Cache.get(key);
                if (rule.canDo()) {
                    this.ruleCheck(rule);
                }
            }
        } catch (Exception e) {
            log.error("RuleCheckTask.checkAll error.", e);
        }
    }

    private void ruleCheck(WarnRule rule) {
        try {
            NSensor sensor = rule.getNSensor();
            ModevTypePoint typePoint = rule.getTypePoint();
            List<ModevTypePoint> points = new ArrayList<>();
            points.add(typePoint);
            List<Map<String, Object>> list = dataService.getLatestData(sensor, points, rule.getLastDTime());
            if (!CollectionUtils.isEmpty(list)) {
                Date date = null;
                for (Map<String, Object> map : list) {
                    String str = (String) map.get("acquisitionTime");
                    date = DateUtil.parse(str);
                    Object value = map.get(typePoint.getField());
                    boolean r = rule.triggerRule(value);
                    if (r) {
                        this.sendWarning(rule, date, str, String.valueOf(value));
                    }
                }
                rule.setLastDTime(date);
                ruleRepository.save(rule);
            }
        } catch (Exception e) {
            log.error("RuleCheckTask.ruleCheck error.", e);
        }
    }

    private void sendWarning(WarnRule rule, Date date, String dateStr, String value) throws Exception {
        NSensor sensor = rule.getNSensor();
        ModevTypePoint typePoint = rule.getTypePoint();
        Warning warning = Warning.builder()
                .sensorId(sensor.getId())
                .sensorName(sensor.getName())
                .zsbName(sensor.getZsbName())
                .ruleId(rule.getId())
                .field(typePoint.getField())
                .dTime(date)
                .threshold(rule.getThreshold())
                .warnValue(value)
                .warnLevel(rule.getLevel())
                .state("1")
                .warnDesc(sensor.getName() + " " + typePoint.getFieldDesc() + " 告警")
                .triggerDesc(rule.getOperatorDesc() + " 阈值(" + rule.getThreshold() + ")")
                .warnTime(new Date())
                .processTime(new Date())
                .build();
        warningRepository.save(warning);
        String str = warning.getZsbName() + "-" + warning.getWarnDesc()
                + ", 采集时间:" + dateStr + ", 当前值:" + value + " " + warning.getTriggerDesc();
        log.warn("触发规则告警: " + str);
        webSocketServer.sendMessage(str);
        dingTalkPushUtil.pushText(str);
    }

}
